### Empty
{
	type => "op",
	name => "unconditionalMatch",
	param => "",
	input => "",
	ret => 1,
},
{
	type => "op",
	name => "unconditionalMatch",
	param => "TestCase",
	input => "",
	ret => 1,
},
{
	type => "op",
	name => "unconditionalMatch",
	param => "",
	input => "TestCase",
	ret => 1,
},

